import { Injectable } from '@angular/core';
import {
  NavigationCancel,
  NavigationEnd,
  NavigationError,
  NavigationStart,
  Router
} from '@angular/router';
import { BehaviorSubject, Observable } from 'rxjs';
import { filter } from 'rxjs/operators';

@Injectable({
  providedIn: 'root'
})
export class MonkeyEcxProgressBarService {
  private visible: BehaviorSubject<any> | null = null;

  constructor(private router: Router) {
    this.init();
  }

  private init() {
    this.visible = new BehaviorSubject(false);

    this.router.events
      .pipe(
        filter((event: any) => {
          return event instanceof NavigationStart;
        })
      )
      .subscribe(() => {
        this.show();
      });

    this.router.events
      .pipe(
        filter((event: any) => {
          return (
            event instanceof NavigationEnd ||
            event instanceof NavigationError ||
            event instanceof NavigationCancel
          );
        })
      )
      .subscribe(() => {
        this.hide();
      });
  }

  show() {
    this.visible?.next(true);
  }

  hide() {
    this.visible?.next(false);
  }

  visibleConfig(): Observable<any> | null {
    return this.visible?.asObservable() || null;
  }
}
