/* eslint-disable no-console */
/* eslint-disable space-before-function-paren */
/* eslint-disable func-names */
/* eslint-disable max-len */
import { MonkeyEcxServiceCredentials } from '../interfaces';
import { DecoratorsUtils } from '../utils';
import { MonkeyEcxUtils } from '../utils/utils';

export function MonkeyEcxCoreServiceConstructor() {
  return (target: any) => {
    Object.entries(target.prototype).forEach(([key, value]) => {
      const method = target.prototype[key];
      if (key !== 'constructor') {
        target.prototype[key] = function (...args: any[]) {
          const context = this;
          if (DecoratorsUtils.hasMonkeyEcxServiceAndHandlingProperties(context)) {
            context.monkeyecxService.monkeyecxHandlingService.clearAll();
          }
          const result = method.apply(context, args);
          return result;
        };
      }
    });
    return target;
  };
}

export function MonkeyEcxCoreLog(tp: string) {
  return function (
    target: Object,
    propertyName: string,
    propertyDescriptor: PropertyDescriptor
  ): PropertyDescriptor {
    const method = propertyDescriptor.value;
    propertyDescriptor.value = function (...args: any[]) {
      const params = args
        .map((a: any) => {
          return JSON.stringify(a);
        })
        .join();
      const result = method.apply(this, args);
      const r = JSON.stringify(result);
      console.log(`Call: ${propertyName}(${params}) => ${r}`);
      return result;
    };

    return propertyDescriptor;
  };
}

export function MonkeyEcxCoreClearDecorators() {
  return function (
    target: Object,
    propertyName: string,
    propertyDescriptor: PropertyDescriptor
  ): PropertyDescriptor {
    const method = propertyDescriptor.value;

    propertyDescriptor.value = function (...args: any[]) {
      const context: any = this;
      if (DecoratorsUtils.hasMonkeyEcxServiceAndHandlingProperties(context)) {
        context?.monkeyecxService?.monkeyecxHandlingService?.clearAll();
      }
      const result = method.apply(context, args);
      return result;
    };

    return propertyDescriptor;
  };
}

export function MonkeyEcxCoreService(params: MonkeyEcxServiceCredentials) {
  return function (
    target: Object,
    propertyName: string,
    propertyDescriptor: PropertyDescriptor
  ): PropertyDescriptor {
    const method = propertyDescriptor.value;

    propertyDescriptor.value = function (...args: any[]) {
      const context: any = this;
      if (DecoratorsUtils.hasMonkeyEcxServiceAndHandlingProperties(context)) {
        context?.monkeyecxService?.monkeyecxHandlingService?.clearAll();
        context?.monkeyecxService?.monkeyecxHandlingService?.setMonkeyEcxServiceCredentials(params);
      }
      const result = method.apply(context, args);
      return result;
    };

    return propertyDescriptor;
  };
}

export function MonkeyEcxCoreServicePaged(params: MonkeyEcxServiceCredentials) {
  return function (
    target: Object,
    propertyName: string,
    propertyDescriptor: PropertyDescriptor
  ): PropertyDescriptor {
    const method = propertyDescriptor.value;

    propertyDescriptor.value = function (...args: any[]) {
      const context: any = this;
      if (DecoratorsUtils.hasMonkeyEcxServiceAndHandlingProperties(context)) {
        const credentials =
          context?.monkeyecxService?.monkeyecxHandlingService?.getMonkeyEcxServiceCredentials();
        let requestPaged = null;
        if (credentials) {
          requestPaged = credentials.requestPaged;
        }
        context?.monkeyecxService?.monkeyecxHandlingService?.clearAll();
        context?.monkeyecxService?.monkeyecxHandlingService?.setMonkeyEcxServiceCredentials({
          ...params,
          ...(requestPaged || {}),
        });
      }
      const result = method.apply(context, args);
      return result;
    };

    return propertyDescriptor;
  };
}

export function MonkeyEcxCoreServiceQueue(
  name: string,
  description?: string,
  params?: MonkeyEcxServiceCredentials
) {
  return function (
    target: Object,
    propertyName: string,
    propertyDescriptor: PropertyDescriptor
  ): PropertyDescriptor {
    const method = propertyDescriptor.value;

    propertyDescriptor.value = function (...args: any[]) {
      const context: any = this;
      if (DecoratorsUtils.hasMonkeyEcxServiceAndHandlingProperties(context)) {
        if (MonkeyEcxUtils.persistNullEmptyUndefined(params)) {
          context?.monkeyecxService?.monkeyecxHandlingService?.clearAll();
          context?.monkeyecxService?.monkeyecxHandlingService?.setMonkeyEcxServiceCredentials(
            params
          );
        }
        context?.monkeyecxService.monkeyecxHandlingService.setMonkeyEcxRequestQueue(
          DecoratorsUtils.buildQueuePropertys({
            name,
            description,
          })
        );
      }
      const result = method.apply(context, args);
      return result;
    };

    return propertyDescriptor;
  };
}
