import { ChangeDetectorRef, Directive, ElementRef, Input, OnInit } from '@angular/core';
import { MonkeyEcxConfig } from '../interfaces';
import { MonkeyEcxConfigService } from '../services';
import { MonkeyEcxUtils } from '../utils/utils';

@Directive({
  selector: '[monkeyecxFeatureByProgram]'
})
export class MonkeyEcxFeatureByProgramDirective implements OnInit {
  @Input('feature') feature: string = '';

  private config?: MonkeyEcxConfig;

  constructor(
    private cdr: ChangeDetectorRef,
    private elementRef: ElementRef,
    private monkeyConfigService: MonkeyEcxConfigService
  ) {
    this.monkeyConfigService.config().subscribe((_: MonkeyEcxConfig) => {
      this.config = _;
    });
  }

  private handleDisplay(): void {
    const { feature, config } = this;
    let display = 'none';

    if (!MonkeyEcxUtils.persistNullEmptyUndefined(feature) ||
    !MonkeyEcxUtils.persistNullEmptyUndefined(config?.program?.settings)) {
      return;
    }

    const disabled = config?.program?.settings?.[feature];

    if (!disabled) {
      display = 'block';
    }

    this.elementRef.nativeElement.style.display = display;

    if (display === 'none') {
      this.elementRef.nativeElement.remove();
    }

    this.cdr.detectChanges();
  }

  ngOnInit() {
    if (!this.feature) return;
    this.cdr.detectChanges();
    this.handleDisplay();
  }
}
