import { ChangeDetectorRef, Directive, ElementRef, Input, OnDestroy, OnInit } from '@angular/core';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { MonkeyEcxUtils } from '../utils/utils';
import { MonkeyEcxFeatureToggleService } from '../services/config/monkeyecx-feature-toggle.service';

@Directive({
  selector: '[monkeyecxFeature]'
})
export class MonkeyEcxFeatureDirective implements OnDestroy, OnInit {
  @Input('featureName') feature?: string;

  private unsubscribeAll = new Subject();

  constructor(
    private cdr: ChangeDetectorRef,
    private elementRef: ElementRef,
    private monkeyecxFeatureToggleService: MonkeyEcxFeatureToggleService
  ) {
    // not to do
  }

  private getFeature(feature: string): boolean {
    const { monkeyecxFeatureToggleService } = this;

    if (!monkeyecxFeatureToggleService) {
      return false;
    }

    return monkeyecxFeatureToggleService?.getFlag(feature) || false;
  }

  private handleDisplay(): void {
    if (!this.feature) return;
    const flag = this.getFeature(this.feature);
    let display = 'none';

    if (!MonkeyEcxUtils.persistNullEmptyUndefined(flag)) {
      return;
    }

    if (flag) {
      display = 'block';
    }

    if (display === 'none') {
      this.elementRef.nativeElement.remove();
    }

    this.elementRef.nativeElement.style.display = display;
    this.cdr.detectChanges();
  }

  ngOnDestroy() {
    this.unsubscribeAll.next();
    this.unsubscribeAll.complete();
  }

  ngOnInit() {
    if (!this.feature) return;
    this.elementRef.nativeElement.style.display = 'none';
    this.cdr.detectChanges();

    this.monkeyecxFeatureToggleService.onFlags
      .pipe(takeUntil(this.unsubscribeAll))
      .subscribe((val: boolean) => {
        if (val) {
          this.handleDisplay();
        }
      });
  }
}
