import { Directive, ElementRef, HostListener, OnInit, Injector } from '@angular/core';
import { CurrencyPipe } from '@angular/common';
import { TranslateService } from '@ngx-translate/core';
import { MonkeyEcxUtils } from '../utils/utils';

@Directive({
  selector: '[monkeyecxFormatCurrency]',
  providers: [CurrencyPipe]
})
export class MonkeyEcxFormatCurrency implements OnInit {
  private el: HTMLInputElement;

  private currency: any;

  private currencyCode: string = 'BRL';

  private currentLocale: string = 'pt-BR';

  constructor(private elementRef: ElementRef, private injector: Injector) {
    this.el = this.elementRef.nativeElement;
    this.currency = this.injector.get(CurrencyPipe);

    const translateService = this.injector.get(TranslateService);
    translateService.get('MONKEY-CURRENCY-CODE').subscribe((_: any) => {
      this.currencyCode = _;
      this.currentLocale = translateService.getDefaultLang();
    });
  }

  private handle(value: any): string {
    const { currencyCode, currentLocale } = this;
    const newValue = MonkeyEcxUtils.handleOnlyNumbers(value);
    return this.currency.transform(newValue / 100, currencyCode, 'symbol', '', currentLocale);
  }

  ngOnInit() {
    this.el.value = this.handle(this.el.value);
  }

  @HostListener('input', ['$event.target.value'])
  onInput(value: any) {
    this.el.value = this.handle(value);
  }
}
