import { ChangeDetectorRef, Directive, ElementRef, Input, OnInit } from '@angular/core';
import { MonkeyEcxTokenCredentials } from '../interfaces';
import { MonkeyEcxTokenStorageService } from '../services/storage';

@Directive({
  selector: '[monkeyecxSecurity]',
})
export class MonkeyEcxSecurityDirective implements OnInit {
  @Input('securityRoles') roles: string[] = [];

  @Input('securityByExclusion') byExclusion: boolean = false;

  private tokenCredentials: MonkeyEcxTokenCredentials | null = null;

  constructor(
    private elementRef: ElementRef,
    private tokenStorageService: MonkeyEcxTokenStorageService,
    private cdr: ChangeDetectorRef
  ) {
    // not to do
  }

  private getRole(): string {
    const { tokenCredentials } = this;

    if (!tokenCredentials) return '';

    if (tokenCredentials?.programAdmin === 'true') {
      return 'PROGRAM_ADMIN';
    }

    return tokenCredentials?.role || '';
  }

  private handleAccess() {
    const { roles, byExclusion } = this;
    let display = 'none';
    const found = roles?.indexOf(this.getRole());
    if (byExclusion) {
      display = 'block';
      if (found > -1) {
        display = 'none';
      }
    } else {
      display = 'none';
      if (found > -1) {
        display = 'block';
      }
    }

    if (!roles || roles.length === 0 || this.getRole() === 'PROGRAM_ADMIN') display = 'block';
    this.elementRef.nativeElement.style.display = display;
    this.cdr.detectChanges();
  }

  ngOnInit() {
    this.elementRef.nativeElement.style.display = 'none';
    this.cdr.detectChanges();

    this.tokenStorageService.tokenHasChanged().subscribe(() => {
      this.tokenCredentials = this.tokenStorageService.getToken();
      this.handleAccess();
    });

    this.tokenCredentials = this.tokenStorageService.getToken();
    this.handleAccess();
  }
}
