import { Directive, Input, ElementRef, HostListener, Renderer2 } from '@angular/core';

@Directive({
  selector: '[monkeyecxTooltip]',
})
export class MonkeyEcxTooltipDirective {
  @Input() tooltipDelay?: number;

  @Input() tooltipPosition?: string;

  @Input() tooltipText?: string;

  @Input() tooltipTitle?: string;

  @Input() enableToShow = true;

  offset = 10;

  tooltip: HTMLElement | null = null;

  constructor(private el: ElementRef, private renderer: Renderer2) {
    // not to do
  }

  @HostListener('mouseenter')
  onMouseEnter() {
    if (!this.tooltip) {
      this.show();
    }
  }

  @HostListener('mouseleave')
  onMouseLeave() {
    if (this.tooltip) {
      this.hide();
    }
  }

  @HostListener('click')
  clickout() {
    if (this.tooltip) {
      this.hide();
    }
  }

  show() {
    if (this.enableToShow) {
      this.create();
      this.setPosition();
      this.renderer.addClass(this.tooltip, 'monkeyecx-tooltip-show');
      setTimeout(() => {
        this.hide();
      }, 5000);
    }
  }

  hide() {
    if (!this.tooltip) return;
    this.renderer.removeClass(this.tooltip, 'monkeyecx-tooltip-show');
    this.renderer.removeChild(document.body, this.tooltip);

    this.tooltip = null;
  }

  create() {
    const tooltipElement = document.createElement('div');

    this.tooltip = this.renderer.createElement('span');

    if (this.tooltipTitle) {
      const tooltipTitleElement = document.createElement('span');

      tooltipTitleElement.innerText = `${this.tooltipTitle}`;
      tooltipTitleElement.className = 'monkeyecx-tooltip-title';

      tooltipElement.appendChild(tooltipTitleElement);
    }

    if (this.tooltipText) {
      const tooltipTextElement = document.createElement('span');

      tooltipTextElement.innerText = `${this.tooltipText}`;
      tooltipTextElement.className = 'monkeyecx-tooltip-description';
      tooltipElement.appendChild(tooltipTextElement);
    }

    this.renderer.appendChild(this.tooltip, tooltipElement);
    this.renderer.appendChild(document.body, this.tooltip);

    this.renderer.addClass(this.tooltip, 'monkeyecx-tooltip');
    this.renderer.addClass(this.tooltip, `monkeyecx-tooltip-${this.tooltipPosition}`);

    this.renderer.setStyle(this.tooltip, '-webkit-transition', `opacity ${this.tooltipDelay}ms`);
    this.renderer.setStyle(this.tooltip, '-moz-transition', `opacity ${this.tooltipDelay}ms`);
    this.renderer.setStyle(this.tooltip, '-o-transition', `opacity ${this.tooltipDelay}ms`);
    this.renderer.setStyle(this.tooltip, 'transition', `opacity ${this.tooltipDelay}ms`);
  }

  setPosition() {
    const hostPos = this.el.nativeElement.getBoundingClientRect();
    const tooltipPos = this.tooltip?.getBoundingClientRect();
    if (!tooltipPos) return;
    const scrollPos =
      window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0;
    let top: number = 0;
    let left: number = 0;

    if (this.tooltipPosition === 'top') {
      top = hostPos.top - tooltipPos.height - this.offset;
      left = hostPos.left + (hostPos.width - tooltipPos.width) / 2;
    }

    if (this.tooltipPosition === 'bottom') {
      top = hostPos.bottom + this.offset;
      left = hostPos.left + (hostPos.width - tooltipPos.width) / 2;
    }

    if (this.tooltipPosition === 'left') {
      top = hostPos.top + (hostPos.height - tooltipPos.height) / 2;
      left = hostPos.left - tooltipPos.width - this.offset;
    }

    if (this.tooltipPosition === 'right') {
      top = hostPos.top + (hostPos.height - tooltipPos.height) / 2;
      left = hostPos.right + this.offset;
    }

    this.renderer.setStyle(this.tooltip, 'top', `${top + scrollPos}px`);
    this.renderer.setStyle(this.tooltip, 'left', `${left}px`);
  }
}
