import { Injectable } from '@angular/core';
import {
  HttpErrorResponse,
  HttpEvent,
  HttpHandler,
  HttpInterceptor,
  HttpRequest
} from '@angular/common/http';
import { Observable, throwError } from 'rxjs';
import { catchError, map, mergeMap, take } from 'rxjs/operators';
import { MonkeyEcxAuthenticationService } from '../services/auth';
import { MonkeyEcxErrorHandlingService } from '../services/error';

export interface Headers {
  name: string;
  value: string;
}

@Injectable()
export class MonkeyEcxHttpConfigErrorInterceptor implements HttpInterceptor {
  constructor(
    private monkeyecxAuthenticationService: MonkeyEcxAuthenticationService,
    private monkeyecxErrorHandlingService: MonkeyEcxErrorHandlingService
  ) {
    // no to do
  }

  intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
    return next.handle(request).pipe(
      map((event: HttpEvent<any>) => {
        return event;
      }),
      catchError((error: HttpErrorResponse) => {
        if (this.monkeyecxAuthenticationService.refreshShouldHappen(error)) {
          return this.monkeyecxAuthenticationService?.refreshToken()?.pipe(
            take(1),
            map(() => {
              return this.monkeyecxAuthenticationService.getRequestWithHeaders(request);
            }),
            mergeMap((resp: any) => {
              return next.handle(resp).pipe(
                catchError((error: any) => {
                  this.monkeyecxErrorHandlingService.handleError(error, 'refresh_token');
                  return throwError(null);
                })
              );
            }),
            catchError((error: any) => {
              this.monkeyecxErrorHandlingService.handleError(error, 'refresh_token');
              return throwError(null);
            })
          ) || throwError(error);
        }
        return throwError(error);
      })
    );
  }
}
