import { Injectable } from '@angular/core';
import {
  HttpErrorResponse,
  HttpEvent,
  HttpHandler,
  HttpInterceptor,
  HttpRequest
} from '@angular/common/http';
import { from, Observable, throwError } from 'rxjs';
import { catchError, first, map, mergeMap, take } from 'rxjs/operators';
import { MonkeyEcxErrorHandlingService } from '../services/error';
import { MonkeyEcxAuthenticationService } from '../services/auth';
import { MonkeyEcxConfigService } from '../services';

@Injectable()
export class MonkeyEcxHttpConfigHeaderInterceptor implements HttpInterceptor {
  constructor(
    private authService: MonkeyEcxAuthenticationService,
    private errorHandlingService: MonkeyEcxErrorHandlingService,
    private config: MonkeyEcxConfigService
  ) {
    // not to do
  }

  private async handle(request: HttpRequest<any>) {
    const { url } = request;

    if (this.authService.isTokenMandatory(url)) {
      try {
        await this.config
          .config()
          .pipe(
            first((val: any) => {
              return !!val && JSON.stringify(val) !== '{}';
            })
          )
          .toPromise();
      } catch (e) {
        // not to do
      }
    }

    const resp = await this.authService
      .getRequestWithHeadersOb(request)
      .pipe(
        take(1),
        map((event: any) => {
          return event;
        }),
        catchError((error: HttpErrorResponse) => {
          this.errorHandlingService.handleError(error);
          return throwError(error);
        })
      )
      .toPromise();

    return resp;
  }

  intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
    return from(this.handle(request)).pipe(
      take(1),
      map((event: any) => {
        return event;
      }),
      mergeMap((resp: any) => {
        request = request.clone({
          setHeaders: resp
        });
        return next.handle(request);
      }),
      catchError((error: HttpErrorResponse) => {
        this.errorHandlingService.handleError(error);
        return throwError(error);
      })
    );
  }
}
