/* eslint-disable max-len */
import { Injectable, Injector } from '@angular/core';
import { HttpEvent, HttpHandler, HttpInterceptor, HttpRequest } from '@angular/common/http';
import { Observable } from 'rxjs';
import { finalize } from 'rxjs/operators';
import { MonkeyEcxHandlingService } from '../services/monkeyecx-handling.service';
import { MonkeyEcxRequestLoadingInProgress } from '../interfaces';
import { MonkeyEcxUtils } from '../utils/utils';
import { MonkeyEcxProgressBarService } from '../../components';

@Injectable()
export class MonkeyEcxHttpConfigLoadingInProgressInterceptor implements HttpInterceptor {
  constructor(private injector: Injector) {
    // no to do
  }

  intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
    const { injector, isLoadingInProgressProperty } = this;
    const handlingService: MonkeyEcxHandlingService = injector.get(MonkeyEcxHandlingService);
    const progressBarService: MonkeyEcxProgressBarService = injector.get(
      MonkeyEcxProgressBarService
    );

    const inProgress: MonkeyEcxRequestLoadingInProgress | null = handlingService?.getMonkeyEcxServiceCredentials()
      ? handlingService?.getMonkeyEcxServiceCredentials()?.requestInProgress || null
      : null;
    if (isLoadingInProgressProperty(inProgress)) {
      progressBarService.show();
    }

    return next.handle(request).pipe(
      finalize(() => {
        if (isLoadingInProgressProperty(inProgress)) {
          progressBarService.hide();
        }
      })
    );
  }

  private isLoadingInProgressProperty(mlp: MonkeyEcxRequestLoadingInProgress | null): boolean {
    if (!mlp) return false;
    return MonkeyEcxUtils.persistNullEmptyUndefined(mlp) &&
      MonkeyEcxUtils.persistNullEmptyUndefined(mlp.showProgress)
      ? mlp.showProgress
      : true;
  }
}
