/* eslint-disable max-len */
import { Injectable, Injector } from '@angular/core';
import { HttpEvent, HttpHandler, HttpInterceptor, HttpRequest } from '@angular/common/http';
import { Observable } from 'rxjs';
import { finalize } from 'rxjs/operators';
import { MonkeyEcxRequestQueueHandlingService } from '../services/request-queue';
import { MonkeyEcxHandlingService } from '../services/monkeyecx-handling.service';
import { MonkeyEcxRequestQueue } from '../interfaces';
import { MonkeyEcxUtils } from '../utils/utils';

@Injectable()
export class MonkeyEcxHttpConfigQueueInterceptor implements HttpInterceptor {
  constructor(
    private injector: Injector,
    private monkeyecxRequestQueueHandlingService: MonkeyEcxRequestQueueHandlingService
  ) {
    // not to do
  }

  intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
    const { injector, isQueueProperty } = this;
    const monkeyecxHandlingService = injector.get(MonkeyEcxHandlingService);

    const monkeyecxRequestQueue: MonkeyEcxRequestQueue | null = monkeyecxHandlingService?.getMonkeyEcxRequestQueue();
    if (isQueueProperty(monkeyecxRequestQueue)) {
      this.monkeyecxRequestQueueHandlingService.setQueue(monkeyecxRequestQueue);
    }

    return next.handle(request).pipe(
      finalize(() => {
        if (isQueueProperty(monkeyecxRequestQueue)) {
          this.monkeyecxRequestQueueHandlingService.finishQueueItem(monkeyecxRequestQueue);
        }
      })
    );
  }

  private isQueueProperty(mrq?: MonkeyEcxRequestQueue | null): boolean {
    return (
      MonkeyEcxUtils.persistNullEmptyUndefined(mrq) &&
      MonkeyEcxUtils.persistNullEmptyUndefined(mrq?.item?.name)
    );
  }
}
