import { Injector, Pipe, PipeTransform } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import * as moment_ from 'moment';
import { MonkeyEcxUtils } from '../utils/utils';

const moment: any = moment_;

@Pipe({
  name: 'monkeyecxFormatDateGroup',
})
export class MonkeyEcxFormatDateGroupPipe implements PipeTransform {
  private lang = 'pt-BR';

  constructor(private injector: Injector) {
    const translateService = this.injector.get(TranslateService);
    this.lang = translateService.getDefaultLang();
  }

  private format(date: string | Date) {
    if (!MonkeyEcxUtils.persistNullEmptyUndefined(date)) return '';
    let stillUtc: any = moment.default.utc(date).toDate();
    if (date.toString()?.indexOf(':') <= -1) {
      stillUtc = date;
    }
    const formatFrom = 'YYYY/MM/DD';
    const fmt = moment.default(stillUtc, formatFrom).locale(this.lang);
    const dayFormated = fmt.format('DD/');
    const monthFormated = MonkeyEcxUtils.capitalize(fmt.format('MMMM'));
    const yearFormated = fmt.format('YYYY');
    return `${`${dayFormated}${monthFormated}`.substring(0, 6)}, ${yearFormated}`;
  }

  transform(obj: any): string {
    if (!obj || !obj?.start || !obj?.end) return '';
    const { start, end } = obj;
    const formatedStart = this.format(start);
    const formatedEnd = this.format(end);
    return `${formatedStart} - ${formatedEnd}`;
  }
}
