/* eslint-disable no-console */
import { Injectable } from '@angular/core';
import { HttpErrorResponse } from '@angular/common/http';
import { throwError, BehaviorSubject, Observable } from 'rxjs';
import { MonkeyStyleGuideSettingsService } from 'monkey-style-guide';
import { MonkeyEcxi18nConfigService } from './monkeyecx-i18n-config.service';
import { MonkeyEcxLogsConfigService } from './monkeyecx-logs-config.service';
import { MonkeyEcxServiceWorkerConfigService } from './monkeyecx-service-worker-config.service';
import { MonkeyEcxSecurityConsoleConfigService } from './monkeyecx-security-console-config.service';
import { MonkeyEcxMaintenanceConfigService } from './monkeyecx-maintenance-config.service';
import { MonkeyEcxCoreService } from '../../decorators/monkeyecx-decorators';
import { MonkeyEcxCommonsService } from '../commons/monkeyecx-commons.service';
import { MonkeyEcxConfig } from '../../interfaces/monkeyecx-config';
import { MonkeyEcxConfigBoostrap } from '../../interfaces/monkeyecx-config-boostrap';
import { MonkeyEcxService } from '../monkeyecx-service.service';
import { MonkeyEcxErrorConfigService } from './monkeyecx-error-config.service';
import { MonkeyEcxFeatureToggleService } from './monkeyecx-feature-toggle.service';
import { MonkeyEcxGAConfigService } from './monkeyecx-ga-config.service';
import { MonkeyEcxAlertsConfigService } from './monkeyecx-alerts-config.service';

@Injectable({
  providedIn: 'root'
})
export class MonkeyEcxConfigService extends MonkeyEcxCommonsService {
  private readonly configSubject$: BehaviorSubject<MonkeyEcxConfig>;

  private readonly configBoostrapSubject$: BehaviorSubject<MonkeyEcxConfig>;

  constructor(
    monkeyecxService: MonkeyEcxService,
    private monkeyecxi18nConfigService: MonkeyEcxi18nConfigService,
    private monkeyecxLogsConfigService: MonkeyEcxLogsConfigService,
    private monkeyStyleGuideSettingsService: MonkeyStyleGuideSettingsService,
    private monkeyecxServiceWorkerConfigService: MonkeyEcxServiceWorkerConfigService,
    private monkeyecxSecurityConsoleConfigService: MonkeyEcxSecurityConsoleConfigService,
    private monkeyecxMaintenanceConfigService: MonkeyEcxMaintenanceConfigService,
    private monkeyecxErrorConfigService: MonkeyEcxErrorConfigService,
    private monkeyEcxFeatureToggleService: MonkeyEcxFeatureToggleService,
    private monkeyGAConfigService: MonkeyEcxGAConfigService,
    private monkeyEcxAlertsConfigService: MonkeyEcxAlertsConfigService
  ) {
    super(monkeyecxService);
    this.configSubject$ = new BehaviorSubject({});
    this.configBoostrapSubject$ = new BehaviorSubject({});
  }

  private internalValidations(...args: any[]) {
    const {
      monkeyecxServiceWorkerConfigService,
      monkeyecxSecurityConsoleConfigService,
      monkeyecxMaintenanceConfigService
    } = this;

    const configBoostrap = args[0];
    const callback = args[1];
    monkeyecxSecurityConsoleConfigService.apply();
    monkeyecxServiceWorkerConfigService.apply(configBoostrap);
    monkeyecxMaintenanceConfigService.apply(configBoostrap);
    callback(configBoostrap);
  }

  private getWhiteLabelSettings(
    monkeyecxCode: string,
    configBoostrap: MonkeyEcxConfigBoostrap,
    callback: Function,
    environment: any,
    identifyCode: string
  ) {
    const {
      monkeyecxService,
      monkeyecxi18nConfigService,
      configSubject$,
      monkeyStyleGuideSettingsService,
      monkeyecxLogsConfigService,
      monkeyecxErrorConfigService,
      monkeyEcxFeatureToggleService,
      monkeyGAConfigService,
      monkeyEcxAlertsConfigService,
      internalValidations
    } = this;

    const url = `${environment.urlAssets}/${monkeyecxCode.toLowerCase()}`;

    monkeyecxService
      ?.get<any>(
        `${url}/white-label.json`
      )
      ?.subscribe(
        (config: MonkeyEcxConfig) => {
          monkeyecxi18nConfigService.apply(config, environment);
          monkeyecxLogsConfigService.apply(config, configBoostrap, environment, identifyCode);
          monkeyEcxFeatureToggleService.apply(configSubject$, environment);
          monkeyGAConfigService.apply(config, environment);
          monkeyEcxAlertsConfigService.apply(config);
          monkeyStyleGuideSettingsService.boostrap(
            `${url}/monkey-style-guide-settings.json`,
            internalValidations.bind(this, configBoostrap, callback, environment)
          );
          configSubject$.next(config);
        },
        (err: HttpErrorResponse) => {
          monkeyecxErrorConfigService.apply('theme');
          console.error('Program Configuration not found!');
          throwError(err);
        }
      );
  }

  @MonkeyEcxCoreService({
    httpResponse: {
      httpCodeIgnore: [404]
    },
    requestInProgress: {
      showProgress: false
    }
  })
  public init(callback: Function, environment: any, identifyCode: string, monkeyecxCode?: string) {
    const { monkeyecxService, configBoostrapSubject$, monkeyecxErrorConfigService } = this;

    monkeyecxService
      ?.get(`${environment.boostrapAssets}`, {
        observe: 'response'
      })
      ?.subscribe(
        (resp: any) => {
          const { headers, body } = resp;
          const monkeyCode = headers.get('monkey-code') || monkeyecxCode;
          if (monkeyCode) {
            this.getWhiteLabelSettings(monkeyCode, body, callback, environment, identifyCode);
            configBoostrapSubject$.next(body);
          } else {
            monkeyecxErrorConfigService.apply('theme');
            console.error(`Boostrap ${monkeyCode} Configuration not found!`);
          }
        },
        (err: HttpErrorResponse) => {
          monkeyecxErrorConfigService.apply('theme');
          console.error('Boostrap Configuration not found!');
          throwError(err);
        }
      );
  }

  public config(): Observable<any> {
    return this.configSubject$.asObservable();
  }

  public boostrapConfig(): Observable<any> {
    return this.configBoostrapSubject$.asObservable();
  }
}
