import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { initialize, LDClient, LDFlagSet } from 'launchdarkly-js-client-sdk';
import { MonkeyUtils } from 'monkey-style-guide';
import { MonkeyEcxConfig } from '../../interfaces';

@Injectable({
  providedIn: 'root',
})
export class MonkeyEcxFeatureToggleService {
  private readonly flagSubject$: BehaviorSubject<any> = new BehaviorSubject(null);

  private flags: LDFlagSet | undefined = undefined;

  private ldClient: LDClient | undefined = undefined;

  constructor() {
    // not to do
  }

  private getAllFlags(): void {
    if (!this) return;
    this.flags = this.ldClient?.allFlags();

    this.flagSubject$.next(true);
  }

  private identify(config: MonkeyEcxConfig, environment: any): void {
    if (config) {
      const email = 'monkey@monkey.exchange';
      const user = {
        email,
        key: `${config.program?.token}#${email}`,
      };

      if (!this.ldClient) {
        this.ldClient = initialize(environment.launchDarklyClientSideID, user, {
          streaming: true,
          disableSyncEventPost: true,
          sendEvents: false,
          diagnosticOptOut: true,
        });

        this.ldClient.on('change', this.getAllFlags.bind(this));
        this.ldClient.on('ready', this.getAllFlags.bind(this));
      } else {
        this.ldClient.identify(user, undefined, this.getAllFlags);
      }
    }
  }

  public apply(configSubject$: BehaviorSubject<MonkeyEcxConfig>, environment: any) {
    if (environment.launchDarklyClientSideID) {
      configSubject$.subscribe((config) => {
        this.identify(config, environment);
      });
    } else {
      this.flagSubject$.next(true);
    }
  }

  public get onFlags(): Observable<any> {
    return this.flagSubject$.asObservable();
  }

  public getFlag(flag: string): boolean | null {
    const flagValue = this.flags?.[flag];

    return MonkeyUtils.persistNullEmptyUndefined(flagValue) ? flagValue : null;
  }
}
