/* eslint-disable no-console */
/* eslint-disable import/no-named-default */
import { Injectable } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { throwError } from 'rxjs';
import { take } from 'rxjs/operators';
import { MonkeyEcxService } from '../monkeyecx-service.service';
import { MonkeyEcxCookieStorageService } from '../storage/monkeyecx-cookie-storage.service';
import { MonkeyEcxCoreService } from '../../decorators/monkeyecx-decorators';
import { MonkeyEcxCommonsService } from '../commons';
import { MonkeyEcxConfig } from '../../interfaces';
import { MonkeyEcxErrorConfigService } from '../config/monkeyecx-error-config.service';

@Injectable({
  providedIn: 'root'
})
export class MonkeyEcxi18nConfigService extends MonkeyEcxCommonsService {
  constructor(
    monkeyecxService: MonkeyEcxService,
    private translateService: TranslateService,
    private monkeyecxCookieStorageService: MonkeyEcxCookieStorageService,
    private monkeyecxErrorConfigService: MonkeyEcxErrorConfigService
  ) {
    super(monkeyecxService);

    this.translateService.addLangs(['pt-BR', 'en-US']);
    this.translateService.setDefaultLang('pt-BR');
    this.translateService.use('pt-BR');
  }

  @MonkeyEcxCoreService({
    httpResponse: {
      httpCodeIgnore: [404]
    },
    requestInProgress: {
      showProgress: false
    }
  })
  public async apply(params: MonkeyEcxConfig, environment: any, change = false) {
    const { monkeyecxService, monkeyecxCookieStorageService, monkeyecxErrorConfigService } = this;
    let { i18n } = params;
    const currentExternali18n = params?.externali18n?.[`${i18n?.lang || 'pt-BR'}`];

    const cookie = monkeyecxCookieStorageService.getCookie('monkey-app-locale');
    if (cookie && !change) {
      i18n = JSON.parse(atob(cookie));
    } else {
      monkeyecxCookieStorageService.setCookie('monkey-app-locale', btoa(JSON.stringify(i18n)), environment);
    }

    try {
      const externali18nPaths: string[] = params?.i18n?.paths || [];

      const services = externali18nPaths?.map((url: string) => {
        return monkeyecxService?.get(
          `${url}/${i18n?.lang}.json`
        ).pipe(take(1)).toPromise();
      });
      if (!services?.length) return;

      const externali18nData = await Promise.all(services);

      externali18nData?.forEach((config: any) => {
        const { lang } = config;
        const { data } = config;
        this.translateService.setTranslation('pt-BR', data, true);
        this.translateService.setTranslation(lang, data, true);
      });
    } catch (err) {
      // not to do
    }

    try {
      const currenti18n: any = await monkeyecxService?.get(
        `${environment.localAssets}/i18n/${i18n?.lang}.json`
      ).pipe(take(1)).toPromise();

      const { lang } = currenti18n;
      let { data } = currenti18n;
      if (currentExternali18n) {
        data = {
          ...data,
          EXTERNAL: currentExternali18n
        };
      }
      this.translateService.setTranslation('pt-BR', data, true);
      this.translateService.setTranslation(lang, data, true);
      this.translateService.use(lang);
    } catch (err) {
      console.error(`i18n ${i18n?.lang} not found!`);
      monkeyecxErrorConfigService.apply('i18n');
      throwError(err);
    }
  }
}
