import { ApplicationRef, Injectable } from '@angular/core';
import { SwUpdate } from '@angular/service-worker';
import { MonkeyStyleGuideModalService } from 'monkey-style-guide';
import { concat, interval } from 'rxjs';
import { first } from 'rxjs/operators';
import { VersionChangedComponent } from '../../../components/version-changed';
import { MonkeyEcxConfigBoostrap } from '../../interfaces';

@Injectable({
  providedIn: 'root'
})
export class MonkeyEcxServiceWorkerConfigService {
  constructor(
    private appRef: ApplicationRef,
    private updates: SwUpdate,
    private modalService: MonkeyStyleGuideModalService) {
    // not to do
  }

  private handleValidation(config: MonkeyEcxConfigBoostrap) {
    const { updates, modalService } = this;
    updates?.available?.subscribe(() => {
      if (!config.showNewVersion || config.maintenance.closed) {
        document.location.reload();
      } else {
        modalService.open(VersionChangedComponent, {
          color: 'theme',
          size: 'md'
        });
      }
    });
  }

  private verify() {
    const { appRef, updates } = this;
    const appIsStable$ = appRef.isStable
      .pipe(first((isStable: boolean) => { return isStable === true; }));
    const everyHour$ = interval(1 * 60 * 60 * 1000);
    const everyHourOnceAppIsStable$ = concat(appIsStable$, everyHour$);
    everyHourOnceAppIsStable$.subscribe(() => {
      updates.checkForUpdate();
    });
    updates.checkForUpdate();
  }

  public apply(config: MonkeyEcxConfigBoostrap) {
    this.handleValidation(config);
    this.verify();
  }
}
