import { Injectable } from '@angular/core';
import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import { Observable, throwError } from 'rxjs';
import { catchError, map } from 'rxjs/operators';
import { MonkeyEcxHandlingService } from './monkeyecx-handling.service';

@Injectable({
  providedIn: 'root',
})
export class MonkeyEcxService {
  constructor(
    protected http: HttpClient,
    public monkeyecxHandlingService: MonkeyEcxHandlingService
  ) {
    // not to do
  }

  private handlelize(params: any): any {
    if (!params) {
      return params;
    }

    // eslint-disable-next-line no-restricted-syntax
    for (const field of [
      'createdAt',
      'createUserId',
      'updatedAt',
      'lastUserId',
      '_links',
      'governmentIdType',
      'governmentIdMask',
      'screenType',
      'screenTab',
    ]) {
      delete params[field];
    }
    return params;
  }

  public get<T>(url: string, options?: T): Observable<T> {
    return this.http.get<T>(url, options);
  }

  public post<T>(url: string, params: T, options?: T): Observable<T> {
    return this.http.post<T>(url, params, options);
  }

  public put<T>(url: string, params: T): Observable<T> {
    return this.http.put<T>(
      url,
      this.handlelize({
        ...params,
      })
    );
  }

  public delete<T>(url: string, params?: T): Observable<T> {
    return this.http.delete<T>(url, params);
  }

  public download<T>(url: string, options?: T, filename?: string) {
    this.http.get<T>(url, options).subscribe(
      (resp: any) => {
        const { headers, body } = resp;
        const contentType = headers.get('Content-Type');
        const contentDisposition = headers.get('Content-Disposition');
        if (contentDisposition) {
          filename = contentDisposition
            .split(';')[1]
            .split('filename')[1]
            .split('=')[1]
            .trim()
            .replace(new RegExp(/"/g), '');
        }
        const blob = new Blob([body], {
          type: `${contentType}`,
        });
        return {
          filename,
          blob,
        };
      },
      (err: HttpErrorResponse) => {
        throwError(err);
      }
    );
  }

  public downloadOb<T>(url: string, options?: T, filename?: string): Observable<any> {
    return this.http.get<any>(url, options).pipe(
      map((resp: any) => {
        const { headers, body } = resp;
        const contentType = headers.get('Content-Type');
        const contentDisposition = headers.get('Content-Disposition');
        if (contentDisposition) {
          filename = contentDisposition
            .split(';')[1]
            .split('filename')[1]
            .split('=')[1]
            .trim()
            .replace(new RegExp(/"/g), '');
        }
        const blob = new Blob([body], {
          type: `${contentType}`,
        });
        return {
          filename,
          blob,
        };
      }),
      catchError((err: any) => {
        return throwError(err);
      })
    );
  }
}
