import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { MonkeyEcxUtils } from '../../utils/utils';
import { MonkeyEcxRequestDownload } from '../../interfaces';
import { MonkeyEcxDownloadEvents } from '../../utils/statics';

@Injectable({
  providedIn: 'root'
})
export class MonkeyEcxRequestDownloadHandlingService {
  private download: MonkeyEcxRequestDownload[] = [];

  private download$: BehaviorSubject<MonkeyEcxRequestDownload[] | any>;

  constructor() {
    this.download$ = new BehaviorSubject([]);
  }

  private addTo(q: MonkeyEcxRequestDownload) {
    this.download.push(q);
    this.download$.next(this.download);
  }

  private markItemAsFinish(q: MonkeyEcxRequestDownload, status?: number) {
    const saved = [...this.download];
    this.download = [
      ...saved
        .map((val: any) => {
          if (val.status === MonkeyEcxDownloadEvents.OnGoing) {
            const valSaved = {
              ...val
            };
            if (valSaved.item.id === q.item.id) {
              return {
                ...valSaved,
                status: status || MonkeyEcxDownloadEvents.Finished
              };
            }
            return valSaved;
          }
          return null;
        })
        .filter((_: any) => {
          return _;
        })
    ];

    this.download = this.download.filter((_: any) => {
      return _.status === MonkeyEcxDownloadEvents.OnGoing;
    });
    this.download$.next(this.download);
  }

  private updateItem(q: MonkeyEcxRequestDownload, loaded?: number, total?: number) {
    const saved = [...this.download];
    this.download = [
      ...saved.map((val: any) => {
        const valSaved = {
          ...val
        };
        const currentPecentage = Math.floor(((loaded || 0) / (total || 0)) * 100);
        const totalPercentage = 100;
        if (valSaved.item.id === q.item.id) {
          return {
            ...valSaved,
            loaded,
            total,
            currentPercentage: currentPecentage,
            totalPercentage
          };
        }
        return valSaved;
      })
    ];
    this.download$.next(this.download);
  }

  public set(q: MonkeyEcxRequestDownload) {
    this.addTo(q);
  }

  public get(): Observable<MonkeyEcxRequestDownload[]> {
    return this.download$.asObservable();
  }

  public update(q: MonkeyEcxRequestDownload, loaded?: number, total?: number) {
    this.updateItem(q, loaded, total);
  }

  public finishItem(q: MonkeyEcxRequestDownload, status?: number) {
    this.markItemAsFinish(q, status);
  }

  public hasDownloadOnGoing(): boolean {
    const found = this.download.find((_: any) => {
      return _.status === MonkeyEcxDownloadEvents.OnGoing;
    });
    return MonkeyEcxUtils.persistNullEmptyUndefined(found);
  }
}
