import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { MonkeyEcxUtils } from '../../utils/utils';
import { Statics } from '../../utils';
import { MonkeyEcxRequestDownloaded } from '../../interfaces';

@Injectable({
  providedIn: 'root'
})
export class MonkeyEcxRequestDownloadedHandlingService {
  private downloaded: MonkeyEcxRequestDownloaded[] = [];

  private downloaded$: BehaviorSubject<MonkeyEcxRequestDownloaded[] | any>;

  constructor() {
    this.downloaded$ = new BehaviorSubject([]);
  }

  private addTo(q: MonkeyEcxRequestDownloaded) {
    this.downloaded.push(q);
    this.downloaded$.next(this.downloaded);
  }

  private markItemAsFinish(q: MonkeyEcxRequestDownloaded, status?: number) {
    const saved = [...this.downloaded];
    this.downloaded = [
      ...saved.map((val: any) => {
        if (val.status === Statics.MonkeyEcxDownloadEvents.OnGoing) {
          const valSaved = {
            ...val
          };
          if (valSaved.item.id === q.item.id) {
            return {
              ...valSaved,
              status: status || Statics.MonkeyEcxDownloadEvents.Finished,
              action: q.action
            };
          }
          return valSaved;
        }
        return val;
      })
    ];
    this.downloaded$.next(this.downloaded);
  }

  public set(q: MonkeyEcxRequestDownloaded) {
    this.addTo(q);
  }

  public get(): Observable<MonkeyEcxRequestDownloaded[]> {
    return this.downloaded$.asObservable();
  }

  public finishItem(q: MonkeyEcxRequestDownloaded, status?: number) {
    this.markItemAsFinish(q, status);
  }

  public hasDownloadOnGoing(): boolean {
    const found = this.downloaded.find((_: any) => {
      return _.status === Statics.MonkeyEcxDownloadEvents.OnGoing;
    });
    return MonkeyEcxUtils.persistNullEmptyUndefined(found);
  }
}
