import { HttpParams } from '@angular/common/http';
import { MonkeyEcxRequestPaged } from '../../interfaces/monkeyecx-request-paged';
import { MonkeyEcxResponseLinks } from '../../interfaces/monkeyecx-response-links';

export class MonkeyEcxRequestPagedHandling {
  private url?: string;

  private pagedParams: MonkeyEcxRequestPaged;

  private links: MonkeyEcxResponseLinks;

  private samePage = false;

  constructor(
    url?: string,
    pagedParams?: MonkeyEcxRequestPaged,
    links?: any,
    samePage: boolean = false
  ) {
    this.url = url;
    this.pagedParams = pagedParams || {
      page: {},
    };
    this.links = links;
    this.samePage = samePage;
  }

  private handlePagedValuesFromApi(url: string = '') {
    if (url.search('\\?') < 0) return url;
    const params = url.substring(url.search('\\?') + 1, url.length);
    url = url.substring(0, url.search('\\?'));
    return `${url}?${new HttpParams({
      fromString: params,
    })
      .delete('size')
      .delete('page')
      .toString()}`;
  }

  private getUrlParams(url: string): string {
    const { pagedParams } = this;
    const { number, size } = pagedParams?.page || {};
    const hasInitialQueryParams = url.search('\\?') > 0;
    return `${!hasInitialQueryParams ? '?' : '&'}page=${number || 0}&size=${size || 20}`;
  }

  public getRequestWithPagedParams(): string {
    const { url, links, samePage } = this;
    if (samePage && links.self) {
      return links.self.href;
    }
    if (links && links.next) {
      return links.next.href;
    }
    const handled = this.handlePagedValuesFromApi(url);
    return `${handled}${this.getUrlParams(handled)}`;
  }
}
