import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { MonkeyEcxRequestProgressQueue, MonkeyEcxRequestQueue } from '../../interfaces';
import { Statics } from '../../utils';

@Injectable({
  providedIn: 'root'
})
export class MonkeyEcxRequestQueueHandlingService {
  private queue: MonkeyEcxRequestQueue[] = [];

  private queues$: BehaviorSubject<MonkeyEcxRequestQueue[] | any>;

  private newQueue = new BehaviorSubject(false);

  constructor() {
    this.queues$ = new BehaviorSubject([]);
  }

  private addToQueue(q: MonkeyEcxRequestQueue) {
    this.queue.push(q);
    this.queues$.next(this.queue);
    this.newQueue.next(true);
  }

  private markQueueItemAsFinish(q: MonkeyEcxRequestQueue, status?: number, action?: Function) {
    const saved = [...this.queue];
    this.queue = [
      ...saved.map((val: any) => {
        const valSaved = {
          ...val
        };
        if (val.status === Statics.MonkeyEcxQueueEvents.OnGoing) {
          if (valSaved.item.id === q.item.id) {
            return {
              ...valSaved,
              status: status || Statics.MonkeyEcxQueueEvents.Finished,
              action
            };
          }
        }
        return valSaved;
      })
    ];
    this.queues$.next(this.queue);
  }

  private updateQueue(q: MonkeyEcxRequestQueue, progress: MonkeyEcxRequestProgressQueue) {
    const saved = [...this.queue];
    this.queue = [
      ...saved.map((val: any) => {
        const valSaved = {
          ...val
        };
        const { total, loaded } = progress;
        const currentPecentage = Math.floor(((loaded||0) / (total || 0)) * 100);
        const totalPercentage = 100;
        if (valSaved.item.id === q.item.id) {
          return {
            ...valSaved,
            progress: {
              loaded,
              total,
              currentPercentage: currentPecentage,
              totalPercentage
            }
          };
        }
        return valSaved;
      })
    ];

    this.queues$.next(this.queue);
  }

  private deleteQueue(q: MonkeyEcxRequestQueue) {
    const saved = [...this.queue];
    this.queue = [
      ...saved.map((val: any) => {
        const valSaved = {
          ...val
        };
        if (valSaved.item.id === q.item.id) {
          return null;
        }
        return valSaved;
      })
    ].filter((_: any) => {
      return _;
    });

    this.queues$.next(this.queue);
  }

  public setQueue(q: MonkeyEcxRequestQueue | null) {
    if (!q) return;
    this.addToQueue(q);
  }

  public getQueue(): Observable<MonkeyEcxRequestQueue[]> {
    return this.queues$.asObservable();
  }

  public finishQueueItem(q: MonkeyEcxRequestQueue | null, action?: Function, status?: number) {
    if (!q) return;
    this.markQueueItemAsFinish(q, status, action);
  }

  public removeQueueItem(q: MonkeyEcxRequestQueue) {
    this.deleteQueue(q);
  }

  public updateQueueItem(q: MonkeyEcxRequestQueue, progress: MonkeyEcxRequestProgressQueue) {
    this.updateQueue(q, progress);
  }

  public hasNewQueue(): Observable<boolean> {
    return this.newQueue.pipe(
      map((_: any) => {
        return _;
      })
    );
  }
}
