import { Injectable } from '@angular/core';
import { HttpErrorResponse } from '@angular/common/http';
import { MonkeyEcxCoreService } from '../../decorators/monkeyecx-decorators';
import { MonkeyEcxUtils } from '../../utils/utils';
import { MonkeyEcxRequestSchedule } from '../../interfaces';
import { MonkeyEcxService } from '../monkeyecx-service.service';
import { MonkeyEcxLoggedHandlingService } from '../logged';

@Injectable({
  providedIn: 'root'
})
export class MonkeyEcxRequestScheduleService {
  private schedule: MonkeyEcxRequestSchedule[] | any[] = [];

  private genericSchedule: MonkeyEcxRequestSchedule[] | any[] = [];

  constructor(
    public monkeyecxService: MonkeyEcxService,
    private monkeyLoggedHandlingService: MonkeyEcxLoggedHandlingService
  ) {
    // not to do
  }

  private addToSchedule(q: MonkeyEcxRequestSchedule) {
    this.schedule.push(q);
    this.monkeyLoggedHandlingService.addSchedule(this.schedule);
  }

  private addToGenericSchedule(q: MonkeyEcxRequestSchedule) {
    this.genericSchedule.push(q);
    this.monkeyLoggedHandlingService.addGenericSchedule(this.genericSchedule);
  }

  private removeFromSchedule(q: MonkeyEcxRequestSchedule) {
    clearInterval(q.interval);
    const saved = [...this.schedule];
    this.schedule = [
      ...saved.map((sch: MonkeyEcxRequestSchedule) => {
        const savedLocal = {
          ...sch
        };
        if (sch.id === q.id) {
          return null;
        }
        return savedLocal;
      })
    ].filter((_: any) => {
      return _;
    });
    this.monkeyLoggedHandlingService.addSchedule(this.schedule);
  }

  private removeFromGenericSchedule(q: MonkeyEcxRequestSchedule) {
    clearInterval(q.interval);
    const saved = [...this.genericSchedule];
    this.genericSchedule = [
      ...saved.map((sch: MonkeyEcxRequestSchedule) => {
        const savedLocal = {
          ...sch
        };
        if (sch.id === q.id) {
          return null;
        }
        return savedLocal;
      })
    ].filter((_: any) => {
      return _;
    });
    this.monkeyLoggedHandlingService.addGenericSchedule(this.genericSchedule);
  }

  private removeFromScheduleById(id: string) {
    const q = this.getScheduleById(id);
    clearInterval(q?.interval);
    const saved = [...this.schedule];
    this.schedule = [
      ...saved.map((sch: MonkeyEcxRequestSchedule) => {
        const savedLocal = {
          ...sch
        };
        if (sch.id === q?.id) {
          return null;
        }
        return savedLocal;
      })
    ].filter((_: any) => {
      return _;
    });
    this.monkeyLoggedHandlingService.addSchedule(this.schedule);
  }

  @MonkeyEcxCoreService({
    httpResponse: {
      httpCodeIgnore: [400, 403, 404, 500, 503]
    },
    requestInProgress: {
      showProgress: false
    }
  })
  private doCall(sch: MonkeyEcxRequestSchedule) {
    const { url, method, params, data, action, errorAction } = sch;

    const errAction = errorAction || ((...args) => {});

    this.monkeyecxService[method.toLowerCase()]<any>(`${url}`, params).subscribe(
      (resp: any) => {
        action(
          {
            ...data,
            ...resp
          },
          sch
        );
      },
      (err: HttpErrorResponse) => {
        this.removeFromSchedule(sch);
        errAction(
          {
            ...data
          },
          sch,
          err
        );
      }
    );
  }

  @MonkeyEcxCoreService({
    httpResponse: {
      httpCodeIgnore: [400, 401, 403, 404, 500, 503],
      httpCodeIgnoreRedirect: [503]
    },
    requestInProgress: {
      showProgress: false
    }
  })
  private doGenericCall(sch: MonkeyEcxRequestSchedule) {
    const { url, method, params, data, action, errorAction } = sch;

    const errAction = errorAction || ((...args) => {});

    this.monkeyecxService[method.toLowerCase()]<any>(`${url}`, params).subscribe(
      (resp: any) => {
        action(
          {
            ...data,
            ...resp
          },
          sch
        );
      },
      (err: HttpErrorResponse) => {
        this.removeFromGenericSchedule(sch);
        errAction(
          {
            ...data
          },
          sch,
          err
        );
      }
    );
  }

  public setSchedule(q: MonkeyEcxRequestSchedule, delay: number = 3000) {
    const id = q?.id || `${MonkeyEcxUtils.getRandomString(40)}`;
    const obj = {
      ...q,
      id
    };

    const interval = setInterval(() => {
      this.doCall({
        ...obj,
        interval
      });
    }, delay);

    const sch = {
      ...obj,
      interval
    };

    this.addToSchedule(sch);
    return sch;
  }

  public setGenericSchedule(q: MonkeyEcxRequestSchedule, delay: number = 3000) {
    const interval = setInterval(() => {
      this.doGenericCall({
        ...q,
        interval
      });
    }, delay);

    const sch = {
      ...q,
      id: `${MonkeyEcxUtils.getRandomString(40)}`,
      interval
    };

    this.addToGenericSchedule(sch);
    return sch;
  }

  public removeSchedule(q: MonkeyEcxRequestSchedule) {
    this.removeFromSchedule(q);
  }

  public removeGenericSchedule(q: MonkeyEcxRequestSchedule) {
    this.removeFromGenericSchedule(q);
  }

  public removeScheduleById(id: string) {
    this.removeFromScheduleById(id);
  }

  public getScheduleById(id: string) {
    const q = this.schedule.find((sch: MonkeyEcxRequestSchedule) => {
      return sch.id === id;
    });
    return q;
  }
}
