/* eslint-disable no-unused-vars */
import { MonkeyEcxUtils } from '../utils/utils';

export enum OpSearch {
  LIKE = ':*:value*',
  BETWEEN = ':first AND :last',
  OR = 'OR',
  AND = 'AND',
  GT = '>:value',
  LT = '<:value',
  EQUAL = '::value',
  IN = ' IN :value',
  NOT = ' !:value'
}

export class MonkeyEcxSpecificationSearch {
  constructor() {
    // not to do
  }

  private handleValue(value: any) {
    return Object.entries(value).map(([key, value1]) => {
      if (!MonkeyEcxUtils.persistNullEmptyUndefined(value1)) {
        return null;
      }
      if (Array.isArray(value1)) {
        if (key === OpSearch.BETWEEN) {
          return `${key}`.replace(':first', value1[0]).replace(':last', value1[1]);
        }
      }
      return `${key}`.replace(':value', `${value1}`);
    })[0];
  }

  private queryVisitor(value: any): any {
    const okay = Object.entries(value).map(([key1, value1]) => {
      if (key1 === OpSearch.OR || key1 === OpSearch.AND) {
        const join = (value1 as any[])
          .map((_: any) => {
            return Object.entries(_).map(([key2, value2]) => {
              if (value2 instanceof Object) {
                value2 = this.handleValue(value2);
              }
              return this.criteriaParser(key2, value2);
            })[0];
          })
          .filter((_: any) => {
            return _;
          })
          .join(` ${key1} `);
        return `${join ? `(${join})` : ''}`;
      }
      if (value1 instanceof Object) {
        value1 = this.handleValue(value1);
      }
      return this.criteriaParser(key1, value1);
    });
    return okay
      .filter((_: any) => {
        return _;
      })
      .join(' AND ');
  }

  private criteriaParser(key: string, value: any | any[]) {
    if (!MonkeyEcxUtils.persistNullEmptyUndefined(value)) {
      return '';
    }
    if (value instanceof Object) {
      return this.queryVisitor(value);
    }
    return `${key}${value}`;
  }

  public build(spec: any): string {
    try {
      return this.queryVisitor(spec);
    } catch (e) {
      throw new Error(`MonkeyEcxSpecificationSearch Error ${e}`);
    }
  }

  public buildMoreThanOne(field: string, op = OpSearch.EQUAL, addSingleQuotes = false) {
    const self: any = this;
    try {
      if (!self[field]) return [];

      const filters = self[field]?.split(',')?.map((filter: string) => {
        if (!filter) return null;
        return {
          [field]: {
            [op]: addSingleQuotes ? `'${filter}'` : `${filter}`
          }
        };
      }).filter((val: any) => { return val; });
      return filters || [];
    } catch (e) {
      throw new Error(`MonkeyEcxSpecificationSearch Error ${e}`);
    }
  }

  public handleFieldSeparatedBy(field: string, signal = ',', op = OpSearch.EQUAL) {
    return this[field]?.split(signal)?.map((_) => {
      return {
        [field]: {
          [op]: _
        }
      };
    });
  }
}
